@@ -201,15 +201,16 @@ class ShaneKastBlueSpectrograph(ShaneKastSpectrograph):
     camera = 'KASTb'
     supported = True
 
-    def get_detector_par(self, hdu, det):
+    def get_detector_par(self, det, hdu=None):
         """
         Return metadata for the selected detector.
 
         Args:
-            hdu (`astropy.io.fits.HDUList`_):
-                The open fits file with the raw image of interest.
             det (:obj:`int`):
                 1-indexed detector number.
+            hdu (`astropy.io.fits.HDUList`_, optional):
+                The open fits file with the raw image of interest.  If not
+                provided, frame-dependent parameters are set to a default.
 
         Returns:
             :class:`~pypeit.images.detector_container.DetectorContainer`:
@@ -217,7 +218,7 @@ class ShaneKastBlueSpectrograph(ShaneKastSpectrograph):
         """
         # Detector 1
         detector_dict = dict(
-            binning=self.get_meta_value(self.get_headarr(hdu), 'binning'),
+            binning='1,1' if hdu is None else self.get_meta_value(self.get_headarr(hdu), 'binning'),
             det=1,
             dataext=0,
             specaxis=1,
@@ -234,14 +235,11 @@ class ShaneKastBlueSpectrograph(ShaneKastSpectrograph):
             ygap=0.,
             ysize=1.,
             darkcurr=0.0,
-            datasec=np.asarray(['[:, 1:1024]', '[:, 1025:2048]']),  # These are rows, columns on the raw frame, 1-indexed
+            # These are rows, columns on the raw frame, 1-indexed
+            datasec=np.asarray(['[:, 1:1024]', '[:, 1025:2048]']),
             oscansec=np.asarray(['[:, 2050:2080]', '[:, 2081:2111]']),
         )
-        # suffix='_blue'
-        detector = detector_container.DetectorContainer(**detector_dict)
-
-        # Return
-        return detector
+        return detector_container.DetectorContainer(**detector_dict)
 
     @classmethod
     def default_pypeit_par(cls):
@@ -330,22 +328,31 @@ class ShaneKastRedSpectrograph(ShaneKastSpectrograph):
     camera = 'KASTr'
     supported = True
 
-    def get_detector_par(self, hdu, det):
+    def get_detector_par(self, det, hdu=None):
         """
         Return metadata for the selected detector.
 
+        .. warning::
+
+            Many of the necessary detector parameters are read from the file
+            header, meaning the ``hdu`` argument is effectively **required** for
+            Shane/KASTr.  The optional use of ``hdu`` is only viable for
+            automatically generated documentation.
+
         Args:
-            hdu (`astropy.io.fits.HDUList`_):
-                The open fits file with the raw image of interest.
             det (:obj:`int`):
                 1-indexed detector number.
+            hdu (`astropy.io.fits.HDUList`_, optional):
+                The open fits file with the raw image of interest.  If not
+                provided, frame-dependent parameters are set to a default.
 
         Returns:
             :class:`~pypeit.images.detector_container.DetectorContainer`:
             Object with the detector metadata.
         """
         # Binning
-        binning = self.get_meta_value(self.get_headarr(hdu), 'binning')  # Could this be detector dependent??
+        # TODO: Could this be detector dependent??
+        binning = '1,1' if hdu is None else self.get_meta_value(self.get_headarr(hdu), 'binning')
 
         # Detector 1
         detector_dict = dict(
@@ -363,8 +370,16 @@ class ShaneKastRedSpectrograph(ShaneKastSpectrograph):
             numamplifiers   = 2,
             gain            = np.atleast_1d([1.9, 1.9]),
             ronoise         = np.atleast_1d([3.8, 3.8]),
+            datasec         = None,
+            oscansec        = None
             )
 
+        if hdu is None:
+            return detector_container.DetectorContainer(**detector_dict)
+
+        # TODO: I don't know how to handle the stuff below for the
+        # auto-generated detector table...
+
         # Parse datasec, oscancsec from the header
         header = hdu[0].header
         naxis1 = header['NAXIS1']
@@ -551,22 +566,24 @@ class ShaneKastRedRetSpectrograph(ShaneKastSpectrograph):
     supported = True
     comment = 'Red reticon'
 
-    def get_detector_par(self, hdu, det):
+    def get_detector_par(self, det, hdu=None):
         """
         Return metadata for the selected detector.
 
         Args:
-            hdu (`astropy.io.fits.HDUList`_):
-                The open fits file with the raw image of interest.
             det (:obj:`int`):
                 1-indexed detector number.
+            hdu (`astropy.io.fits.HDUList`_, optional):
+                The open fits file with the raw image of interest.  If not
+                provided, frame-dependent parameters are set to a default.
 
         Returns:
             :class:`~pypeit.images.detector_container.DetectorContainer`:
             Object with the detector metadata.
         """
         # Binning
-        binning = self.get_meta_value(self.get_headarr(hdu), 'binning')  # Could this be detector dependent??
+        # TODO: Could this be detector dependent??
+        binning = '1,1' if hdu is None else self.get_meta_value(self.get_headarr(hdu), 'binning')
 
         # Detector 1
         detector_dict = dict(
