@@ -2,7 +2,9 @@
 Module to run tests on scripts
 """
 import os
+from pkg_resources import resource_filename
 import shutil
+from pathlib import Path
 
 import numpy as np
 import pytest
@@ -182,14 +184,36 @@ def test_chk_edges():
 
 
 @cooked_required
-def test_view_fits():
-    """ Only test the list option
+def test_view_fits_list():
+    """ Test the list option
     """
     spec_file = os.path.join(os.getenv('PYPEIT_DEV'), 'Cooked', 'Science',
                             'spec1d_b27-J1217p3905_KASTb_20150520T045733.560.fits')
     #spec_file = data_path('spec1d_b27-J1217p3905_KASTb_2015May20T045733.560.fits')
-    pargs = scripts.view_fits.ViewFits.parse_args([spec_file, '--list', 'shane_kast_blue'])
-    # TODO: Should this test be calling the main function?
+    pargs = scripts.view_fits.ViewFits.parse_args(['shane_kast_blue', spec_file, '--list'])
+    scripts.view_fits.ViewFits.main(pargs)
+
+
+@cooked_required
+def test_view_fits_proc_fail():
+    """ Test that it fails when trying to proc an output pypeit image
+    """
+    spec_file = os.path.join(os.getenv('PYPEIT_DEV'), 'Cooked', 'Science',
+                            'spec2d_b27-J1217p3905_KASTb_20150520T045733.560.fits')
+    #spec_file = data_path('spec1d_b27-J1217p3905_KASTb_2015May20T045733.560.fits')
+    pargs = scripts.view_fits.ViewFits.parse_args(['shane_kast_blue', spec_file, '--proc'])
+    with pytest.raises(PypeItError):
+        scripts.view_fits.ViewFits.main(pargs)
+
+
+@dev_suite_required
+def test_view_fits_proc():
+    """ Test that it works on a raw image
+    """
+    spec_file = Path(os.getenv('PYPEIT_DEV')).resolve() / 'RAW_DATA' / 'shane_kast_blue' \
+                    / '830_3460_d46' / 'b100.fits.gz'
+    pargs = scripts.view_fits.ViewFits.parse_args(['shane_kast_blue', str(spec_file), '--proc'])
+    scripts.view_fits.ViewFits.main(pargs)
 
 
 @cooked_required
@@ -327,9 +351,27 @@ def test_obslog():
     # Clean up
     shutil.rmtree(setupdir)
 
+@dev_suite_required
+def test_compare_sky():
+    spec_file = os.path.join(os.getenv('PYPEIT_DEV'), 'Cooked', 'Science',
+                             'spec1d_b27-J1217p3905_KASTb_20150520T045733.560.fits')
+    sky_file = os.path.join(resource_filename('pypeit', 'data/sky_spec/'),
+                                              'sky_kastb_600.fits')
+
+    # Running in `test` mode for boxcar extraction
+    pargs = scripts.compare_sky.CompareSky.parse_args([spec_file, sky_file, '--test'])
+    scripts.compare_sky.CompareSky.main(pargs)
+
+    # Running in `test` mode for optimal extraction
+    pargs = scripts.compare_sky.CompareSky.parse_args([spec_file, sky_file, '--test',
+                                                       '--optimal'])
+    scripts.compare_sky.CompareSky.main(pargs)
+
 @cooked_required
 def test_collate_1d(tmp_path, monkeypatch):
-    args = ['--dry_run', '--archive_dir', '/archive', '--match', 'ra/dec', '--exclude_slit_bm', 'BOXSLIT', '--exclude_serendip']
+
+    # Build up arguments for testing command line parsing
+    args = ['--dry_run', '--archive_dir', '/archive', '--outdir', '/outdir2', '--pypeit_file', 'file.pypeit', '--match', 'ra/dec', '--exclude_slit_bm', 'BOXSLIT', '--exclude_serendip']
     spec1d_file = os.path.join(os.getenv('PYPEIT_DEV'), 'Cooked', 'Science', 'spec1d_b27*')
     spec1d_args = ['--spec1d_files', spec1d_file]
     tol_args = ['--tolerance', '0.03d']
@@ -337,6 +379,8 @@ def test_collate_1d(tmp_path, monkeypatch):
     expanded_spec1d = os.path.join(os.getenv('PYPEIT_DEV'), 'Cooked', 'Science', 'spec1d_b27-J1217p3905_KASTb_20150520T045733.560.fits')
     expanded_alt_spec1d = os.path.join(os.getenv('PYPEIT_DEV'), 'Cooked', 'Science', 'spec1d_DE.20100913.22358-CFHQS1_DEIMOS_20100913T061231.334.fits')
     spec1d_args = ['--spec1d_files', expanded_spec1d]
+
+    # Create config files for testing config file parsing
     config_file_full = str(tmp_path / "test_collate1d_full.collate1d")
 
     with open(config_file_full, "w") as f:
@@ -344,6 +388,8 @@ def test_collate_1d(tmp_path, monkeypatch):
         print("ex_value = BOX", file=f)
         print("[collate1d]", file=f)
         print("dry_run = False", file=f)
+        print("outdir = /outdir", file=f)
+        print("pypeit_file = otherfile.pypeit", file=f)
         print("archive_root = /foo/bar", file=f)
         print("tolerance = 4.0", file=f)
         print("match_using = 'pixel'", file=f)
@@ -365,8 +411,8 @@ def test_collate_1d(tmp_path, monkeypatch):
         print("[coadd1d]", file=f)
         print("ex_value = BOX", file=f)
 
-    # Args only, nospec1d files should raise an exception
-    with pytest.raises(PypeItError):
+    # Args only, nospec1d files should exit with an errror
+    with pytest.raises(SystemExit):
         parsed_args = scripts.collate_1d.Collate1D.parse_args(args + tol_args)
         params, spectrograph, expanded_spec1d_files \
                 = scripts.collate_1d.build_parameters(parsed_args)
@@ -376,6 +422,8 @@ def test_collate_1d(tmp_path, monkeypatch):
     params, spectrograph, expanded_spec1d_files = scripts.collate_1d.build_parameters(parsed_args)
     assert params['collate1d']['dry_run'] is True
     assert params['collate1d']['archive_root'] == '/archive'
+    assert params['collate1d']['outdir'] == '/outdir2'
+    assert params['collate1d']['pypeit_file'] == 'file.pypeit'
     assert params['collate1d']['match_using'] == 'ra/dec'
     assert params['collate1d']['tolerance'] == '0.03d'
     assert params['collate1d']['exclude_slit_trace_bm'] == ['BOXSLIT']
@@ -389,6 +437,8 @@ def test_collate_1d(tmp_path, monkeypatch):
     params, spectrograph, expanded_spec1d_files = scripts.collate_1d.build_parameters(parsed_args)
     assert params['collate1d']['dry_run'] is False
     assert params['collate1d']['archive_root'] == '/foo/bar'
+    assert params['collate1d']['outdir'] == '/outdir'
+    assert params['collate1d']['pypeit_file'] == 'otherfile.pypeit'
     assert params['collate1d']['tolerance'] == 4.0
     assert params['collate1d']['match_using'] == 'pixel'
     assert params['collate1d']['exclude_slit_trace_bm'] == 'BADREDUCE'
@@ -403,6 +453,8 @@ def test_collate_1d(tmp_path, monkeypatch):
     params, spectrograph, expanded_spec1d_files = scripts.collate_1d.build_parameters(parsed_args)
     assert params['collate1d']['dry_run'] is True
     assert params['collate1d']['archive_root'] == '/archive'
+    assert params['collate1d']['outdir'] == '/outdir2'
+    assert params['collate1d']['pypeit_file'] == 'file.pypeit'
     assert params['collate1d']['tolerance'] == '0.03d'
     assert params['collate1d']['match_using'] == 'ra/dec'
     assert params['collate1d']['exclude_slit_trace_bm'] == ['BOXSLIT']
@@ -432,50 +484,87 @@ def test_collate_1d(tmp_path, monkeypatch):
     def mock_get_instance(*args, **kwargs):
         return MockCoadd()
 
+    def mock_get_subdir(*args, **kwargs):
+        return "subdir"
+
     with monkeypatch.context() as m:
         monkeypatch.setattr(coadd1d.CoAdd1D, "get_instance", mock_get_instance)
+        monkeypatch.setattr(scripts.collate_1d, "get_archive_subdir", mock_get_subdir)
+
+        os.chdir(tmp_path)
+        par_file = str(tmp_path / 'collate1d.par')
+        
+        # For testing archiving, create a fake pypeit output directory structure
+        # in temp space.
+        temp_spec1d_dir = tmp_path / 'Science'
+        temp_spec1d_dir.mkdir()
+        # Copy spec1d and spec2d
+        shutil.copy2(expanded_spec1d, temp_spec1d_dir)
+        temp_spec1d = os.path.join(temp_spec1d_dir, os.path.basename(expanded_spec1d))
+
+        spec2d_file = expanded_spec1d.replace("spec1d", "spec2d")
+        shutil.copy2(spec2d_file, temp_spec1d_dir)
+        temp_spec2d = os.path.join(temp_spec1d_dir, os.path.basename(spec2d_file))
+
+        # Create fake text files for archiving, and a fake coadd output. We copy the
+        # the spec1d over to the fake output file because the archiving code reads
+        # the header of the coadd output
+
+        temp_spec1d_text = temp_spec1d.replace(".fits", ".txt")
+        temp_pypeit_file = str(tmp_path / "temp.pypeit")
+        with open(temp_spec1d.replace(".fits", ".txt"), "w") as f:
+            print("test data", file=f)
+        shutil.copy2(temp_spec1d_text, temp_pypeit_file)
+
+        temp_coadd_output = str(tmp_path / "SPAT0176-SLIT0175-DET01_KASTb_20150520.fits")
+        shutil.copy2(temp_spec1d, temp_coadd_output)
 
         # Test:
         # * main
         # * creation of collate1d.par
         # * parsing of pixel tolerance
         # * detection of spec2d files and excluding by slit bitmask
+        # * archiving of spec1ds, spec2ds, .txt files, and pypeit files
+
+        archive_dir = tmp_path / 'archive'
 
-        os.chdir(tmp_path)
-        par_file = str(tmp_path / 'collate1d.par')
         parsed_args = scripts.collate_1d.Collate1D.parse_args(['--par_outfile', par_file, '--match',
                                                                'pixel', '--tolerance', '3',
-                                                               config_file_spec1d,
-                                                               '--exclude_slit_bm', 'BADREDUCE'])
+                                                               '--spec1d_files', temp_spec1d,
+                                                               '--exclude_slit_bm', 'BADREDUCE', 
+                                                               '--archive_dir', str(archive_dir)])
         assert scripts.collate_1d.Collate1D.main(parsed_args) == 0
         assert os.path.exists(par_file)
+
+        archive_dest_dir = archive_dir / "subdir"
+        assert os.path.exists(archive_dest_dir / os.path.basename(temp_coadd_output))
+        assert os.path.exists(archive_dest_dir / os.path.basename(temp_spec1d))
+        assert os.path.exists(archive_dest_dir / os.path.basename(temp_spec2d))
+        assert os.path.exists(archive_dest_dir / os.path.basename(temp_spec1d_text))
+        assert os.path.exists(archive_dest_dir / os.path.basename(temp_pypeit_file))
+
         # Remove par_file to avoid a warning
         os.unlink(par_file)
         
         # Test:
         # * default units of arcsec for tolerance when match is ra/dec
         # * that a spec2d file isn't needed if exclude_slit_flags is empty.
-        # * test specifying archive dir, including copying a file to it
         # * test exception handling when one file fails
         # The 240 arsec tolerance is to ensure there's only two outputs, one of which the mock 
         # coadd object will fail
-        coadd_output = "J232913.02-030531.05_DEIMOS_20100913.fits"
-        with open(coadd_output, "w") as f:
-            print("test data", file=f)
-        archive_dir = tmp_path / 'archive'
         parsed_args = scripts.collate_1d.Collate1D.parse_args(['--par_outfile', par_file,
                                                                '--match', 'ra/dec', '--tolerance',
                                                                '240', '--spec1d_files',
-                                                               expanded_alt_spec1d,
-                                                               '--archive_dir', str(archive_dir)])
+                                                               expanded_alt_spec1d])                                                            
         assert scripts.collate_1d.Collate1D.main(parsed_args) == 0
-        assert os.path.exists(archive_dir / coadd_output)
+
+        # Remove par_file to avoid a warning
+        os.unlink(par_file)
 
         # Test parsing of units in ra/dec tolerance
         parsed_args = scripts.collate_1d.Collate1D.parse_args(['--par_outfile', par_file,
-                                                               '--match', 'ra/dec', '--tolerance',
-                                                               '3d', '--spec1d_files',
-                                                               expanded_alt_spec1d])
+                                                               '--match', 'ra/dec', '--tolerance', '3d',
+                                                               '--spec1d_files', expanded_alt_spec1d])
         assert scripts.collate_1d.Collate1D.main(parsed_args) == 0
         
 
